@@ -415,7 +415,7 @@ export const applyRestEvent = async (event, socket, navigate, params) => {
     if (event.payload.files === undefined || event.payload.files.length === 0) {
       // Submit the event over the websocket to trigger the event handler.
       return await applyEvent(
-        Event(event.name, { files: [] }),
+        ReflexEvent(event.name, { files: [] }),
         socket,
         navigate,
         params,
@@ -530,6 +530,7 @@ export const connect = async (
     transports: transports,
     protocols: [reflexEnvironment.version],
     autoUnref: false,
+    query: { token: getToken() },
   });
   // Ensure undefined fields in events are sent as null instead of removed
   socket.current.io.encoder.replacer = (k, v) => (v === undefined ? null : v);
@@ -601,6 +602,10 @@ export const connect = async (
     event_processing = false;
     queueEvents([...initialEvents(), event], socket, true, navigate, params);
   });
+  socket.current.on("new_token", async (new_token) => {
+    token = new_token;
+    window.sessionStorage.setItem(TOKEN_KEY, new_token);
+  });
 
   document.addEventListener("visibilitychange", checkVisibility);
 };
@@ -770,7 +775,7 @@ export const uploadFiles = async (
  * @param {string} handler The client handler to process event.
  * @returns The event object.
  */
-export const Event = (
+export const ReflexEvent = (
   name,
   payload = {},
   event_actions = {},
@@ -964,17 +969,7 @@ export const useEventLoop = (
   useEffect(() => {
     if (!sentHydrate.current) {
       queueEvents(
-        initial_events().map((e) => ({
-          ...e,
-          router_data: {
-            pathname: location.pathname,
-            query: {
-              ...Object.fromEntries(searchParams.entries()),
-              ...params.current,
-            },
-            asPath: location.pathname + location.search,
-          },
-        })),
+        initial_events(),
         socket,
         true,
         navigate,
@@ -992,7 +987,7 @@ export const useEventLoop = (
 
     window.onerror = function (msg, url, lineNo, columnNo, error) {
       addEvents([
-        Event(`${exception_state_name}.handle_frontend_exception`, {
+        ReflexEvent(`${exception_state_name}.handle_frontend_exception`, {
           info: error.name + ": " + error.message + "\n" + error.stack,
           component_stack: "",
         }),
@@ -1004,7 +999,7 @@ export const useEventLoop = (
     //https://github.com/mknichel/javascript-errors?tab=readme-ov-file#promise-rejection-events
     window.onunhandledrejection = function (event) {
       addEvents([
-        Event(`${exception_state_name}.handle_frontend_exception`, {
+        ReflexEvent(`${exception_state_name}.handle_frontend_exception`, {
           info:
             event.reason?.name +
             ": " +
@@ -1077,7 +1072,7 @@ export const useEventLoop = (
       if (storage_to_state_map[e.key]) {
         const vars = {};
         vars[storage_to_state_map[e.key]] = e.newValue;
-        const event = Event(
+        const event = ReflexEvent(
           `${state_name}.reflex___state____update_vars_internal_state.update_vars_internal`,
           { vars: vars },
         );
