@@ -22,7 +22,7 @@ from typing_extensions import dataclass_transform
 
 import reflex.state
 from reflex import constants
-from reflex.compiler.templates import STATEFUL_COMPONENT
+from reflex.compiler.templates import stateful_component_template
 from reflex.components.core.breakpoints import Breakpoints
 from reflex.components.dynamic import load_dynamic_serializer
 from reflex.components.field import BaseField, FieldBasedMeta
@@ -37,7 +37,7 @@ from reflex.constants import (
     PageNames,
 )
 from reflex.constants.compiler import SpecialAttributes
-from reflex.constants.state import FRONTEND_EVENT_STATE
+from reflex.constants.state import FRONTEND_EVENT_STATE, MEMO_MARKER
 from reflex.event import (
     EventCallback,
     EventChain,
@@ -51,7 +51,7 @@ from reflex.event import (
 )
 from reflex.style import Style, format_as_emotion
 from reflex.utils import console, format, imports, types
-from reflex.utils.imports import ImportDict, ImportVar, ParsedImportDict, parse_imports
+from reflex.utils.imports import ImportDict, ImportVar, ParsedImportDict
 from reflex.vars import VarData
 from reflex.vars.base import (
     CachedVarOperation,
@@ -419,7 +419,6 @@ def evaluate_style_namespaces(style: ComponentStyle) -> dict:
 
 # Map from component to styling.
 ComponentStyle = dict[str | type[BaseComponent] | Callable | ComponentNamespace, Any]
-ComponentChild = types.PrimitiveType | Var | BaseComponent
 ComponentChildTypes = (*types.PrimitiveTypes, Var, BaseComponent, type(None))
 
 
@@ -480,7 +479,21 @@ def _components_from(
     return ()
 
 
-def _deterministic_hash(value: object) -> int:
+def _hash_str(value: str) -> str:
+    return md5(f'"{value}"'.encode(), usedforsecurity=False).hexdigest()
+
+
+def _hash_sequence(value: Sequence) -> str:
+    return _hash_str(str([_deterministic_hash(v) for v in value]))
+
+
+def _hash_dict(value: dict) -> str:
+    return _hash_sequence(
+        sorted([(k, _deterministic_hash(v)) for k, v in value.items()])
+    )
+
+
+def _deterministic_hash(value: object) -> str:
     """Hash a rendered dictionary.
 
     Args:
@@ -492,37 +505,28 @@ def _deterministic_hash(value: object) -> int:
     Raises:
         TypeError: If the value is not hashable.
     """
-    if isinstance(value, BaseComponent):
-        # If the value is a component, hash its rendered code.
-        rendered_code = value.render()
-        return _deterministic_hash(rendered_code)
-    if isinstance(value, Var):
-        return _deterministic_hash((value._js_expr, value._get_all_var_data()))
-    if isinstance(value, VarData):
-        return _deterministic_hash(dataclasses.asdict(value))
-    if isinstance(value, dict):
-        # Sort the dictionary to ensure consistent hashing.
-        return _deterministic_hash(
-            tuple(sorted((k, _deterministic_hash(v)) for k, v in value.items()))
-        )
-    if isinstance(value, int):
-        # Hash numbers and booleans directly.
-        return int(value)
-    if isinstance(value, float):
-        return _deterministic_hash(str(value))
-    if isinstance(value, str):
-        return int(md5(f'"{value}"'.encode()).hexdigest(), 16)
-    if isinstance(value, (tuple, list)):
-        # Hash tuples by hashing each element.
-        return _deterministic_hash(
-            "[" + ",".join(map(str, map(_deterministic_hash, value))) + "]"
-        )
-    if isinstance(value, enum.Enum):
-        # Hash enums by their name.
-        return _deterministic_hash(str(value))
     if value is None:
         # Hash None as a special case.
-        return _deterministic_hash("None")
+        return "None"
+    if isinstance(value, (int, float, enum.Enum)):
+        # Hash numbers and booleans directly.
+        return str(value)
+    if isinstance(value, str):
+        return _hash_str(value)
+    if isinstance(value, dict):
+        return _hash_dict(value)
+    if isinstance(value, (tuple, list)):
+        # Hash tuples by hashing each element.
+        return _hash_sequence(value)
+    if isinstance(value, Var):
+        return _hash_str(
+            str((value._js_expr, _deterministic_hash(value._get_all_var_data())))
+        )
+    if isinstance(value, VarData):
+        return _hash_dict(dataclasses.asdict(value))
+    if isinstance(value, BaseComponent):
+        # If the value is a component, hash its rendered code.
+        return _hash_dict(value.render())
 
     msg = (
         f"Cannot hash value `{value}` of type `{type(value).__name__}`. "
@@ -1017,44 +1021,8 @@ class Component(BaseComponent, ABC):
         """
         return set()
 
-    @classmethod
-    def _are_fields_known(cls) -> bool:
-        """Check if all fields are known at compile time. True for most components.
-
-        Returns:
-            Whether all fields are known at compile time.
-        """
-        return True
-
-    @classmethod
-    @functools.cache
-    def _get_component_prop_names(cls) -> set[str]:
-        """Get the names of the component props. NOTE: This assumes all fields are known.
-
-        Returns:
-            The names of the component props.
-        """
-        return {
-            name
-            for name in cls.get_fields()
-            if name in cls.get_props()
-            and types._issubclass(
-                types.value_inside_optional(types.get_field_type(cls, name)), Component
-            )
-        }
-
-    def _get_components_in_props(self) -> Sequence[BaseComponent]:
-        """Get the components in the props.
-
-        Returns:
-            The components in the props
-        """
-        if self._are_fields_known():
-            return [
-                component
-                for name in self._get_component_prop_names()
-                for component in _components_from(getattr(self, name))
-            ]
+    @functools.cached_property
+    def _get_component_prop_property(self) -> Sequence[BaseComponent]:
         return [
             component
             for prop in self.get_props()
@@ -1063,6 +1031,14 @@ class Component(BaseComponent, ABC):
             for component in _components_from(value)
         ]
 
+    def _get_components_in_props(self) -> Sequence[BaseComponent]:
+        """Get the components in the props.
+
+        Returns:
+            The components in the props
+        """
+        return self._get_component_prop_property
+
     @classmethod
     def _validate_children(cls, children: tuple | list):
         from reflex.utils.exceptions import ChildrenTypeError
@@ -1270,7 +1246,6 @@ class Component(BaseComponent, ABC):
         rendered_dict = dict(
             tag.set(
                 children=[child.render() for child in self.children],
-                contents=str(tag.contents),
             )
         )
         self._replace_prop_names(rendered_dict)
@@ -1498,7 +1473,7 @@ class Component(BaseComponent, ABC):
             yield clz.__name__
 
     @classmethod
-    def _iter_parent_classes_with_method(cls, method: str) -> Iterator[type[Component]]:
+    def _iter_parent_classes_with_method(cls, method: str) -> Sequence[type[Component]]:
         """Iterate through parent classes that define a given method.
 
         Used for handling the `add_*` API functions that internally simulate a super() call chain.
@@ -1506,12 +1481,14 @@ class Component(BaseComponent, ABC):
         Args:
             method: The method to look for.
 
-        Yields:
-            The parent classes that define the method (differently than the base).
+        Returns:
+            A sequence of parent classes that define the method (differently than the base).
         """
+        current_class_method = getattr(Component, method, None)
         seen_methods = (
-            {getattr(Component, method)} if hasattr(Component, method) else set()
+            {current_class_method} if current_class_method is not None else set()
         )
+        clzs: list[type[Component]] = []
         for clz in cls.mro():
             if clz is Component:
                 break
@@ -1521,7 +1498,8 @@ class Component(BaseComponent, ABC):
             if not callable(method_func) or method_func in seen_methods:
                 continue
             seen_methods.add(method_func)
-            yield clz
+            clzs.append(clz)
+        return clzs
 
     def _get_custom_code(self) -> str | None:
         """Get custom code for the component.
@@ -1644,18 +1622,18 @@ class Component(BaseComponent, ABC):
         Returns:
             The imports needed by the component.
         """
-        _imports = {}
-
-        # Import this component's tag from the main library.
-        if self.library is not None and self.tag is not None:
-            _imports[self.library] = self.import_var
+        _imports = (
+            {self.library: [self.import_var]}
+            if self.library is not None and self.tag is not None
+            else {}
+        )
 
         # Get static imports required for event processing.
         event_imports = Imports.EVENTS if self.event_triggers else {}
 
         # Collect imports from Vars used directly by this component.
         var_imports = [
-            var_data.imports
+            dict(var_data.imports)
             for var in self._get_vars()
             if (var_data := var._get_all_var_data()) is not None
         ]
@@ -1665,16 +1643,16 @@ class Component(BaseComponent, ABC):
             list_of_import_dict = clz.add_imports(self)
 
             if not isinstance(list_of_import_dict, list):
-                list_of_import_dict = [list_of_import_dict]
+                added_import_dicts.append(imports.parse_imports(list_of_import_dict))
+            else:
+                added_import_dicts.extend(
+                    [imports.parse_imports(item) for item in list_of_import_dict]
+                )
 
-            added_import_dicts.extend(
-                [parse_imports(import_dict) for import_dict in list_of_import_dict]
-            )
-
-        return imports.merge_imports(
+        return imports.merge_parsed_imports(
             self._get_dependencies_imports(),
             self._get_hooks_imports(),
-            {**_imports},
+            _imports,
             event_imports,
             *var_imports,
             *added_import_dicts,
@@ -1689,7 +1667,7 @@ class Component(BaseComponent, ABC):
         Returns:
             The import dict with the required imports.
         """
-        _imports = imports.merge_imports(
+        _imports = imports.merge_parsed_imports(
             self._get_imports(), *[child._get_all_imports() for child in self.children]
         )
         return imports.collapse_imports(_imports) if collapse else _imports
@@ -1806,7 +1784,7 @@ class Component(BaseComponent, ABC):
 
         # Add the hook code from add_hooks for each parent class (this is reversed to preserve
         # the order of the hooks in the final output)
-        for clz in reversed(tuple(self._iter_parent_classes_with_method("add_hooks"))):
+        for clz in reversed(self._iter_parent_classes_with_method("add_hooks")):
             for hook in clz.add_hooks(self):
                 if isinstance(hook, Var):
                     extract_var_hooks(hook)
@@ -2005,7 +1983,7 @@ class CustomComponent(Component):
 
         super()._post_init(
             event_triggers={
-                key: EventChain.create(
+                key + MEMO_MARKER: EventChain.create(
                     value=props[key],
                     args_spec=get_args_spec(key),
                     key=key,
@@ -2016,7 +1994,9 @@ class CustomComponent(Component):
         )
 
         to_camel_cased_props = {
-            format.to_camel_case(key) for key in props if key not in event_types
+            format.to_camel_case(key + MEMO_MARKER)
+            for key in props
+            if key not in event_types
         }
         self.get_props = lambda: to_camel_cased_props  # pyright: ignore [reportIncompatibleVariableOverride]
 
@@ -2031,7 +2011,7 @@ class CustomComponent(Component):
             if key not in props_types:
                 continue
 
-            camel_cased_key = format.to_camel_case(key)
+            camel_cased_key = format.to_camel_case(key + MEMO_MARKER)
 
             # Get the type based on the annotation.
             type_ = props_types[key]
@@ -2048,15 +2028,6 @@ class CustomComponent(Component):
             self.props[camel_cased_key] = value
             setattr(self, camel_cased_key, value)
 
-    @classmethod
-    def _are_fields_known(cls) -> bool:
-        """Check if the fields are known.
-
-        Returns:
-            Whether the fields are known.
-        """
-        return False
-
     def __eq__(self, other: Any) -> bool:
         """Check if the component is equal to another.
 
@@ -2187,6 +2158,9 @@ def _register_custom_component(
     Args:
         component_fn: The function that creates the component.
 
+    Returns:
+        The custom component.
+
     Raises:
         TypeError: If the tag name cannot be determined.
     """
@@ -2211,6 +2185,7 @@ def _register_custom_component(
         msg = f"Could not determine the tag name for {component_fn!r}"
         raise TypeError(msg)
     CUSTOM_COMPONENTS[dummy_component.tag] = dummy_component
+    return dummy_component
 
 
 def custom_component(
@@ -2234,7 +2209,24 @@ def custom_component(
         )
 
     # Register this component so it can be compiled.
-    _register_custom_component(component_fn)
+    dummy_component = _register_custom_component(component_fn)
+    if tag := dummy_component.tag:
+        object.__setattr__(
+            wrapper,
+            "_as_var",
+            lambda: Var(
+                tag,
+                _var_type=type[Component],
+                _var_data=VarData(
+                    imports={
+                        f"$/{constants.Dirs.UTILS}/components": [ImportVar(tag=tag)],
+                        "@emotion/react": [
+                            ImportVar(tag="jsx"),
+                        ],
+                    }
+                ),
+            ),
+        )
 
     return wrapper
 
@@ -2454,7 +2446,7 @@ class StatefulComponent(BaseComponent):
             return None
 
         # Compute the hash based on the rendered code.
-        code_hash = _deterministic_hash(rendered_code)
+        code_hash = _hash_str(_deterministic_hash(rendered_code))
 
         # Format the tag name including the hash.
         return format.format_state_name(
@@ -2468,7 +2460,7 @@ class StatefulComponent(BaseComponent):
         if not self.tag:
             return ""
         # Render the code for this component and hooks.
-        return STATEFUL_COMPONENT.render(
+        return stateful_component_template(
             tag_name=self.tag,
             memo_trigger_hooks=self.memo_trigger_hooks,
             component=self.component,
@@ -2596,7 +2588,7 @@ class StatefulComponent(BaseComponent):
             memo_name = f"{event_trigger}_{chain_hash}"
 
             # Calculate Var dependencies accessed by the handler for useCallback dep array.
-            var_deps = ["addEvents", "Event"]
+            var_deps = ["addEvents", "ReflexEvent"]
 
             # Get deps from event trigger var data.
             var_deps.extend(cls._get_deps_from_event_trigger(event))
@@ -2794,6 +2786,9 @@ def render_dict_to_var(tag: dict | Component | str) -> Var:
             return render_dict_to_var(tag.render())
         return Var.create(tag)
 
+    if "contents" in tag:
+        return Var(tag["contents"])
+
     if "iterable" in tag:
         function_return = LiteralArrayVar.create(
             [render_dict_to_var(child.render()) for child in tag["children"]]
@@ -2811,27 +2806,30 @@ def render_dict_to_var(tag: dict | Component | str) -> Var:
             func,
         )
 
-    if tag["name"] == "match":
-        element = tag["cond"]
+    if "match_cases" in tag:
+        element = Var(tag["cond"])
 
         conditionals = render_dict_to_var(tag["default"])
 
         for case in tag["match_cases"][::-1]:
-            condition = case[0].to_string() == element.to_string()
-            for pattern in case[1:-1]:
-                condition = condition | (pattern.to_string() == element.to_string())
+            conditions, return_value = case
+            condition = Var.create(False)
+            for pattern in conditions:
+                condition = condition | (
+                    Var(pattern).to_string() == element.to_string()
+                )
 
             conditionals = ternary_operation(
                 condition,
-                render_dict_to_var(case[-1]),
+                render_dict_to_var(return_value),
                 conditionals,
             )
 
         return conditionals
 
-    if "cond" in tag:
+    if "cond_state" in tag:
         return ternary_operation(
-            tag["cond"],
+            Var(tag["cond_state"]),
             render_dict_to_var(tag["true_value"]),
             render_dict_to_var(tag["false_value"])
             if tag["false_value"] is not None
@@ -2840,8 +2838,6 @@ def render_dict_to_var(tag: dict | Component | str) -> Var:
 
     props = Var("({" + ",".join(tag["props"]) + "})")
 
-    contents = tag["contents"] if tag["contents"] else None
-
     raw_tag_name = tag.get("name")
     tag_name = Var(raw_tag_name or "Fragment")
 
@@ -2850,7 +2846,6 @@ def render_dict_to_var(tag: dict | Component | str) -> Var:
     ).call(
         tag_name,
         props,
-        *([Var(contents)] if contents is not None else []),
         *[render_dict_to_var(child) for child in tag["children"]],
     )
 
