@@ -6,7 +6,7 @@ from reflex.utils.imports import ImportVar
 from reflex.vars.base import LiteralVar, Var
 from reflex.vars.sequence import LiteralStringVar, StringVar
 
-LUCIDE_LIBRARY = "lucide-react@0.534.0"
+LUCIDE_LIBRARY = "lucide-react@0.541.0"
 
 
 class LucideIconComponent(Component):
@@ -93,6 +93,7 @@ class DynamicIcon(LucideIconComponent):
     tag = "DynamicIcon"
 
     name: Var[str]
+    size: Var[int]
 
     def _get_imports(self):
         _imports = super()._get_imports()
@@ -342,6 +343,7 @@ LUCIDE_ICON_LIST = [
     "brain_cog",
     "brain",
     "brick_wall_fire",
+    "brick_wall_shield",
     "brick_wall",
     "briefcase_business",
     "briefcase_conveyor_belt",
@@ -451,6 +453,7 @@ LUCIDE_ICON_LIST = [
     "chevrons_up_down",
     "chevrons_up",
     "chrome",
+    "chromium",
     "church",
     "cigarette_off",
     "cigarette",
@@ -493,6 +496,7 @@ LUCIDE_ICON_LIST = [
     "circle_slash_2",
     "circle_slash",
     "circle_small",
+    "circle_star",
     "circle_stop",
     "circle_user_round",
     "circle_user",
@@ -874,6 +878,7 @@ LUCIDE_ICON_LIST = [
     "hamburger",
     "hammer",
     "hand_coins",
+    "hand_fist",
     "hand_heart",
     "hand_helping",
     "hand_metal",
@@ -948,6 +953,7 @@ LUCIDE_ICON_LIST = [
     "japanese_yen",
     "joystick",
     "kanban",
+    "kayak",
     "key_round",
     "key_square",
     "key",
@@ -1193,11 +1199,13 @@ LUCIDE_ICON_LIST = [
     "panel_left_close",
     "panel_left_dashed",
     "panel_left_open",
+    "panel_left_right_dashed",
     "panel_left",
     "panel_right_close",
     "panel_right_dashed",
     "panel_right_open",
     "panel_right",
+    "panel_top_bottom_dashed",
     "panel_top_close",
     "panel_top_dashed",
     "panel_top_open",
@@ -1325,6 +1333,7 @@ LUCIDE_ICON_LIST = [
     "rocket",
     "rocking_chair",
     "roller_coaster",
+    "rose",
     "rotate_3d",
     "rotate_ccw_key",
     "rotate_ccw_square",
@@ -1644,6 +1653,7 @@ LUCIDE_ICON_LIST = [
     "truck_electric",
     "truck",
     "turkish_lira",
+    "turntable",
     "turtle",
     "tv_2",
     "tv_minimal_play",
