@@ -1,21 +1,18 @@
 """The Reflex config."""
 
-from __future__ import annotations
-
+import dataclasses
 import importlib
 import os
 import sys
 import threading
 import urllib.parse
+from collections.abc import Sequence
 from importlib.util import find_spec
 from pathlib import Path
 from types import ModuleType
-from typing import Any, ClassVar
-
-import pydantic.v1 as pydantic
+from typing import TYPE_CHECKING, Any, ClassVar
 
 from reflex import constants
-from reflex.base import Base
 from reflex.constants.base import LogLevel
 from reflex.environment import EnvironmentVariables as EnvironmentVariables
 from reflex.environment import EnvVar as EnvVar
@@ -28,12 +25,16 @@ from reflex.environment import (
 from reflex.environment import env_var as env_var
 from reflex.environment import environment as environment
 from reflex.plugins import Plugin
+from reflex.plugins.sitemap import SitemapPlugin
 from reflex.utils import console
 from reflex.utils.exceptions import ConfigError
-from reflex.utils.types import true_type_for_pydantic_field
+
+if TYPE_CHECKING:
+    from pyleak.base import LeakAction
 
 
-class DBConfig(Base):
+@dataclasses.dataclass(kw_only=True)
+class DBConfig:
     """Database config."""
 
     engine: str
@@ -51,7 +52,7 @@ class DBConfig(Base):
         password: str | None = None,
         host: str | None = None,
         port: int | None = 5432,
-    ) -> DBConfig:
+    ) -> "DBConfig":
         """Create an instance with postgresql engine.
 
         Args:
@@ -81,7 +82,7 @@ class DBConfig(Base):
         password: str | None = None,
         host: str | None = None,
         port: int | None = 5432,
-    ) -> DBConfig:
+    ) -> "DBConfig":
         """Create an instance with postgresql+psycopg engine.
 
         Args:
@@ -107,7 +108,7 @@ class DBConfig(Base):
     def sqlite(
         cls,
         database: str,
-    ) -> DBConfig:
+    ) -> "DBConfig":
         """Create an instance with sqlite engine.
 
         Args:
@@ -145,32 +146,9 @@ class DBConfig(Base):
 _sensitive_env_vars = {"DB_URL", "ASYNC_DB_URL", "REDIS_URL"}
 
 
-class Config(Base):
-    """The config defines runtime settings for the app.
-
-    By default, the config is defined in an `rxconfig.py` file in the root of the app.
-
-    ```python
-    # rxconfig.py
-    import reflex as rx
-
-    config = rx.Config(
-        app_name="myapp",
-        api_url="http://localhost:8000",
-    )
-    ```
-
-    Every config value can be overridden by an environment variable with the same name in uppercase.
-    For example, `db_url` can be overridden by setting the `DB_URL` environment variable.
-
-    See the [configuration](https://reflex.dev/docs/getting-started/configuration/) docs for more info.
-    """
-
-    class Config:  # pyright: ignore [reportIncompatibleVariableOverride]
-        """Pydantic config for the config."""
-
-        validate_assignment = True
-        use_enum_values = False
+@dataclasses.dataclass(kw_only=True)
+class BaseConfig:
+    """Base config for the Reflex app."""
 
     # The name of the app (should match the name of the app directory).
     app_name: str
@@ -211,6 +189,18 @@ class Config(Base):
     # Telemetry opt-in.
     telemetry_enabled: bool = True
 
+    # PyLeak monitoring configuration for detecting event loop blocking and resource leaks.
+    enable_pyleak_monitoring: bool = False
+
+    # Threshold in seconds for detecting event loop blocking operations.
+    pyleak_blocking_threshold: float = 0.1
+
+    # Grace period in seconds for thread leak detection cleanup.
+    pyleak_thread_grace_period: float = 0.2
+
+    # Action to take when PyLeak detects issues
+    pyleak_action: "LeakAction | None" = None
+
     # The bun path
     bun_path: ExistingPath = constants.Bun.DEFAULT_PATH
 
@@ -218,13 +208,13 @@ class Config(Base):
     static_page_generation_timeout: int = 60
 
     # List of origins that are allowed to connect to the backend API.
-    cors_allowed_origins: list[str] = ["*"]
+    cors_allowed_origins: Sequence[str] = dataclasses.field(default=("*",))
 
     # Whether to use React strict mode.
     react_strict_mode: bool = True
 
     # Additional frontend packages to install.
-    frontend_packages: list[str] = []
+    frontend_packages: list[str] = dataclasses.field(default_factory=list)
 
     # Indicate which type of state manager to use
     state_manager_mode: constants.StateManagerMode = constants.StateManagerMode.DISK
@@ -239,13 +229,15 @@ class Config(Base):
     redis_token_expiration: int = constants.Expiration.TOKEN
 
     # Attributes that were explicitly set by the user.
-    _non_default_attributes: set[str] = pydantic.PrivateAttr(set())
+    _non_default_attributes: set[str] = dataclasses.field(
+        default_factory=set, init=False
+    )
 
     # Path to file containing key-values pairs to override in the environment; Dotenv format.
     env_file: str | None = None
 
     # Whether to automatically create setters for state base vars
-    state_auto_setters: bool = True
+    state_auto_setters: bool | None = None
 
     # Whether to display the sticky "Built with Reflex" badge on all pages.
     show_built_with_reflex: bool | None = None
@@ -257,21 +249,91 @@ class Config(Base):
     extra_overlay_function: str | None = None
 
     # List of plugins to use in the app.
-    plugins: list[Plugin] = []
+    plugins: list[Plugin] = dataclasses.field(default_factory=list)
+
+    # List of fully qualified import paths of plugins to disable in the app (e.g. reflex.plugins.sitemap.SitemapPlugin).
+    disable_plugins: list[str] = dataclasses.field(default_factory=list)
+
+    # Whether to skip plugin checks.
+    _skip_plugins_checks: bool = dataclasses.field(default=False, repr=False)
 
     _prefixes: ClassVar[list[str]] = ["REFLEX_"]
 
-    def __init__(self, *args, **kwargs):
-        """Initialize the config values.
+
+_PLUGINS_ENABLED_BY_DEFAULT = [
+    SitemapPlugin,
+]
+
+
+@dataclasses.dataclass(kw_only=True, init=False)
+class Config(BaseConfig):
+    """Configuration class for Reflex applications.
+
+    The config defines runtime settings for your app including server ports, database connections,
+    frontend packages, and deployment settings.
+
+    By default, the config is defined in an `rxconfig.py` file in the root of your app:
+
+    ```python
+    # rxconfig.py
+    import reflex as rx
+
+    config = rx.Config(
+        app_name="myapp",
+        # Server configuration
+        frontend_port=3000,
+        backend_port=8000,
+        # Database
+        db_url="postgresql://user:pass@localhost:5432/mydb",
+        # Additional frontend packages
+        frontend_packages=["react-icons"],
+        # CORS settings for production
+        cors_allowed_origins=["https://mydomain.com"],
+    )
+    ```
+
+    ## Environment Variable Overrides
+
+    Any config value can be overridden by setting an environment variable with the `REFLEX_`
+    prefix and the parameter name in uppercase:
+
+    ```bash
+    REFLEX_DB_URL="postgresql://user:pass@localhost/db" reflex run
+    REFLEX_FRONTEND_PORT=3001 reflex run
+    ```
+
+    ## Key Configuration Areas
+
+    - **App Settings**: `app_name`, `loglevel`, `telemetry_enabled`
+    - **Server**: `frontend_port`, `backend_port`, `api_url`, `cors_allowed_origins`
+    - **Database**: `db_url`, `async_db_url`, `redis_url`
+    - **Frontend**: `frontend_packages`, `react_strict_mode`
+    - **State Management**: `state_manager_mode`, `state_auto_setters`
+    - **Plugins**: `plugins`, `disable_plugins`
+
+    See the [configuration docs](https://reflex.dev/docs/advanced-onboarding/configuration) for complete details on all available options.
+    """
+
+    # Track whether the app name has already been validated for this Config instance.
+    _app_name_is_valid: bool = dataclasses.field(default=False, repr=False)
+
+    def _post_init(self, **kwargs):
+        """Post-initialization method to set up the config.
+
+        This method is called after the config is initialized. It sets up the
+        environment variables, updates the config from the environment, and
+        replaces default URLs if ports were set.
 
         Args:
-            *args: The args to pass to the Pydantic init method.
-            **kwargs: The kwargs to pass to the Pydantic init method.
+            **kwargs: The kwargs passed to the Pydantic init method.
 
         Raises:
             ConfigError: If some values in the config are invalid.
         """
-        super().__init__(*args, **kwargs)
+        class_fields = self.class_fields()
+        for key, value in kwargs.items():
+            if key not in class_fields:
+                setattr(self, key, value)
 
         # Clean up this code when we remove plain envvar in 0.8.0
         env_loglevel = os.environ.get("REFLEX_LOGLEVEL")
@@ -285,9 +347,13 @@ class Config(Base):
         for key, env_value in env_kwargs.items():
             setattr(self, key, env_value)
 
+        # Add builtin plugins if not disabled.
+        if not self._skip_plugins_checks:
+            self._add_builtin_plugins()
+
         #   Update default URLs if ports were set
         kwargs.update(env_kwargs)
-        self._non_default_attributes.update(kwargs)
+        self._non_default_attributes = set(kwargs.keys())
         self._replace_defaults(**kwargs)
 
         if (
@@ -297,6 +363,74 @@ class Config(Base):
             msg = f"{self._prefixes[0]}REDIS_URL is required when using the redis state manager."
             raise ConfigError(msg)
 
+    def _add_builtin_plugins(self):
+        """Add the builtin plugins to the config."""
+        for plugin in _PLUGINS_ENABLED_BY_DEFAULT:
+            plugin_name = plugin.__module__ + "." + plugin.__qualname__
+            if plugin_name not in self.disable_plugins:
+                if not any(isinstance(p, plugin) for p in self.plugins):
+                    console.warn(
+                        f"`{plugin_name}` plugin is enabled by default, but not explicitly added to the config. "
+                        "If you want to use it, please add it to the `plugins` list in your config inside of `rxconfig.py`. "
+                        f"To disable this plugin, set `disable_plugins` to `{[plugin_name, *self.disable_plugins]!r}`.",
+                    )
+                    self.plugins.append(plugin())
+            else:
+                if any(isinstance(p, plugin) for p in self.plugins):
+                    console.warn(
+                        f"`{plugin_name}` is disabled in the config, but it is still present in the `plugins` list. "
+                        "Please remove it from the `plugins` list in your config inside of `rxconfig.py`.",
+                    )
+
+        for disabled_plugin in self.disable_plugins:
+            if not isinstance(disabled_plugin, str):
+                console.warn(
+                    f"reflex.Config.disable_plugins should only contain strings, but got {disabled_plugin!r}. "
+                )
+            if not any(
+                plugin.__module__ + "." + plugin.__qualname__ == disabled_plugin
+                for plugin in _PLUGINS_ENABLED_BY_DEFAULT
+            ):
+                console.warn(
+                    f"`{disabled_plugin}` is disabled in the config, but it is not a built-in plugin. "
+                    "Please remove it from the `disable_plugins` list in your config inside of `rxconfig.py`.",
+                )
+
+    @classmethod
+    def class_fields(cls) -> set[str]:
+        """Get the fields of the config class.
+
+        Returns:
+            The fields of the config class.
+        """
+        return {field.name for field in dataclasses.fields(cls)}
+
+    if not TYPE_CHECKING:
+
+        def __init__(self, **kwargs):
+            """Initialize the config values.
+
+            Args:
+                **kwargs: The kwargs to pass to the Pydantic init method.
+
+            # noqa: DAR101 self
+            """
+            class_fields = self.class_fields()
+            super().__init__(**{k: v for k, v in kwargs.items() if k in class_fields})
+            self._post_init(**kwargs)
+
+    def json(self) -> str:
+        """Get the config as a JSON string.
+
+        Returns:
+            The config as a JSON string.
+        """
+        import json
+
+        from reflex.utils.serializers import serialize
+
+        return json.dumps(self, default=serialize)
+
     @property
     def app_module(self) -> ModuleType | None:
         """Return the app module if `app_module_import` is set.
@@ -333,11 +467,13 @@ class Config(Base):
 
         updated_values = {}
         # Iterate over the fields.
-        for key, field in self.__fields__.items():
+        for field in dataclasses.fields(self):
             # The env var name is the key in uppercase.
             environment_variable = None
             for prefix in self._prefixes:
-                if environment_variable := os.environ.get(f"{prefix}{key.upper()}"):
+                if environment_variable := os.environ.get(
+                    f"{prefix}{field.name.upper()}"
+                ):
                     break
 
             # If the env var is set, override the config value.
@@ -345,19 +481,19 @@ class Config(Base):
                 # Interpret the value.
                 value = interpret_env_var_value(
                     environment_variable,
-                    true_type_for_pydantic_field(field),
+                    field.type,
                     field.name,
                 )
 
                 # Set the value.
-                updated_values[key] = value
+                updated_values[field.name] = value
 
-                if key.upper() in _sensitive_env_vars:
+                if field.name.upper() in _sensitive_env_vars:
                     environment_variable = "***"
 
-                if value != getattr(self, key):
+                if value != getattr(self, field.name):
                     console.debug(
-                        f"Overriding config value {key} with env var {key.upper()}={environment_variable}",
+                        f"Overriding config value {field.name} with env var {field.name.upper()}={environment_variable}",
                         dedupe=True,
                     )
         return updated_values
@@ -429,7 +565,7 @@ def _get_config() -> Config:
     if not spec:
         # we need this condition to ensure that a ModuleNotFound error is not thrown when
         # running unit/integration tests or during `reflex init`.
-        return Config(app_name="")
+        return Config(app_name="", _skip_plugins_checks=True)
     rxconfig = importlib.import_module(constants.Config.MODULE)
     return rxconfig.config
 
