@@ -6,7 +6,6 @@ import concurrent.futures
 import dataclasses
 import enum
 import importlib
-import inspect
 import multiprocessing
 import os
 import platform
@@ -159,7 +158,7 @@ def interpret_plugin_env(value: str, field_name: str) -> Plugin:
         msg = f"Failed to get plugin class {plugin_name!r} from module {import_path!r} for {field_name}: {e}"
         raise EnvironmentVarValueError(msg) from e
 
-    if not inspect.isclass(plugin_class) or not issubclass(plugin_class, Plugin):
+    if not isinstance(plugin_class, type) or not issubclass(plugin_class, Plugin):
         msg = f"Invalid plugin class: {plugin_name!r} for {field_name}. Must be a subclass of Plugin."
         raise EnvironmentVarValueError(msg)
 
@@ -236,7 +235,7 @@ def interpret_env_var_value(
             )
             for i, v in enumerate(value.split(":"))
         ]
-    if inspect.isclass(field_type) and issubclass(field_type, enum.Enum):
+    if isinstance(field_type, type) and issubclass(field_type, enum.Enum):
         return interpret_enum_env(value, field_type, field_name)
 
     msg = f"Invalid type for environment variable {field_name}: {field_type}. This is probably an issue in Reflex."
@@ -640,6 +639,12 @@ class EnvironmentVariables:
     # Enable full logging of debug messages to reflex user directory.
     REFLEX_ENABLE_FULL_LOGGING: EnvVar[bool] = env_var(False)
 
+    # Whether to enable hot module replacement
+    VITE_HMR: EnvVar[bool] = env_var(True)
+
+    # Whether to force a full reload on changes.
+    VITE_FORCE_FULL_RELOAD: EnvVar[bool] = env_var(False)
+
 
 environment = EnvironmentVariables()
 
