@@ -8,6 +8,7 @@ import os
 import signal
 import socket
 import subprocess
+import sys
 from collections.abc import Callable, Generator, Sequence
 from concurrent import futures
 from contextlib import closing
@@ -53,7 +54,7 @@ def get_num_workers() -> int:
     return (os.cpu_count() or 1) * 2 + 1
 
 
-def _is_address_responsive(
+def _can_bind_at_port(
     address_family: socket.AddressFamily | int, address: str, port: int
 ) -> bool:
     """Check if a given address and port are responsive.
@@ -68,45 +69,53 @@ def _is_address_responsive(
     """
     try:
         with closing(socket.socket(address_family, socket.SOCK_STREAM)) as sock:
-            return sock.connect_ex((address, port)) == 0
-    except OSError:
+            if sys.platform != "win32":
+                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            sock.bind((address, port))
+    except (OverflowError, PermissionError, OSError) as e:
+        console.warn(f"Unable to bind to {address}:{port} due to: {e}.")
+        return False
+    return True
+
+
+def _can_bind_at_any_port(address_family: socket.AddressFamily | int) -> bool:
+    """Check if any port is available for binding.
+
+    Args:
+        address_family: The address family (e.g., socket.AF_INET or socket.AF_INET6).
+
+    Returns:
+        Whether any port is available for binding.
+    """
+    try:
+        with closing(socket.socket(address_family, socket.SOCK_STREAM)) as sock:
+            sock.bind(("", 0))  # Bind to any available port
+            return True
+    except (OverflowError, PermissionError, OSError) as e:
+        console.debug(f"Unable to bind to any port for {address_family}: {e}")
         return False
 
 
-def is_process_on_port(port: int) -> bool:
+def is_process_on_port(
+    port: int,
+    address_families: Sequence[socket.AddressFamily | int] = (
+        socket.AF_INET,
+        socket.AF_INET6,
+    ),
+) -> bool:
     """Check if a process is running on the given port.
 
     Args:
         port: The port.
+        address_families: The address families to check (default: IPv4 and IPv6).
 
     Returns:
         Whether a process is running on the given port.
     """
-    return _is_address_responsive(  # Test IPv4 localhost (127.0.0.1)
-        socket.AF_INET, "127.0.0.1", port
-    ) or _is_address_responsive(
-        socket.AF_INET6, "::1", port
-    )  # Test IPv6 localhost (::1)
+    return any(not _can_bind_at_port(family, "", port) for family in address_families)
 
 
-def change_port(port: int, _type: str) -> int:
-    """Change the port.
-
-    Args:
-        port: The port.
-        _type: The type of the port.
-
-    Returns:
-        The new port.
-
-    """
-    new_port = port + 1
-    if is_process_on_port(new_port):
-        return change_port(new_port, _type)
-    console.info(
-        f"The {_type} will run on port [bold underline]{new_port}[/bold underline]."
-    )
-    return new_port
+MAXIMUM_PORT = 2**16 - 1
 
 
 def handle_port(service_name: str, port: int, auto_increment: bool) -> int:
@@ -125,13 +134,45 @@ def handle_port(service_name: str, port: int, auto_increment: bool) -> int:
         Exit:when the port is in use.
     """
     console.debug(f"Checking if {service_name.capitalize()} port: {port} is in use.")
-    if not is_process_on_port(port):
+
+    families = [
+        address_family
+        for address_family in (socket.AF_INET, socket.AF_INET6)
+        if _can_bind_at_any_port(address_family)
+    ]
+
+    if not families:
+        console.error(
+            f"Unable to bind to any port for {service_name}. "
+            "Please check your network configuration."
+        )
+        raise click.exceptions.Exit(1)
+
+    console.debug(
+        f"Checking if {service_name.capitalize()} port: {port} is in use for families: {families}."
+    )
+
+    if not is_process_on_port(port, families):
         console.debug(f"{service_name.capitalize()} port: {port} is not in use.")
         return port
+
     if auto_increment:
-        return change_port(port, service_name)
-    console.error(f"{service_name.capitalize()} port: {port} is already in use.")
-    raise click.exceptions.Exit
+        for new_port in range(port + 1, MAXIMUM_PORT + 1):
+            if not is_process_on_port(new_port, families):
+                console.info(
+                    f"The {service_name} will run on port [bold underline]{new_port}[/bold underline]."
+                )
+                return new_port
+            console.debug(
+                f"{service_name.capitalize()} port: {new_port} is already in use."
+            )
+
+        # If we reach here, it means we couldn't find an available port.
+        console.error(f"Unable to find an available port for {service_name}")
+    else:
+        console.error(f"{service_name.capitalize()} port: {port} is already in use.")
+
+    raise click.exceptions.Exit(1)
 
 
 @overload
