@@ -52,11 +52,8 @@ UnionTypes = (Union, types.UnionType)
 GenericType = type | _GenericAlias
 
 # Valid state var types.
-JSONType = {str, int, float, bool}
-PrimitiveType = int | float | bool | str | list | dict | set | tuple
 PrimitiveTypes = (int, float, bool, str, list, dict, set, tuple)
-StateVar = PrimitiveType | Base | None
-StateIterVar = list | set | tuple
+StateVarTypes = (*PrimitiveTypes, Base, type(None))
 
 if TYPE_CHECKING:
     from reflex.vars.base import Var
@@ -401,6 +398,8 @@ def get_field_type(cls: GenericType, field_name: str) -> GenericType | None:
     Returns:
         The type of the field, if it exists, else None.
     """
+    if (fields := getattr(cls, "_fields", None)) is not None and field_name in fields:
+        return fields[field_name].annotated_type
     if (
         hasattr(cls, "__fields__")
         and field_name in cls.__fields__
@@ -857,8 +856,15 @@ def is_valid_var_type(type_: type) -> bool:
 
     if is_union(type_):
         return all(is_valid_var_type(arg) for arg in get_args(type_))
+
+    if is_literal(type_):
+        types = {type(value) for value in get_args(type_)}
+        return all(is_valid_var_type(type_) for type_ in types)
+
+    type_ = origin if (origin := get_origin(type_)) is not None else type_
+
     return (
-        _issubclass(type_, StateVar)
+        issubclass(type_, StateVarTypes)
         or serializers.has_serializer(type_)
         or dataclasses.is_dataclass(type_)
     )
@@ -993,11 +999,6 @@ def validate_literal(key: str, value: Any, expected_type: type, comp_name: str):
             raise ValueError(msg)
 
 
-# Store this here for performance.
-StateBases = get_base_class(StateVar)
-StateIterBases = get_base_class(StateIterVar)
-
-
 def safe_issubclass(cls: Any, cls_check: Any | tuple[Any, ...]):
     """Check if a class is a subclass of another class. Returns False if internal error occurs.
 
